

#include "SkillProviderExample.h"

#include <RobotAPI/libraries/aron/core/type/variant/container/Object.h>
#include <RobotAPI/libraries/aron/core/type/variant/primitive/String.h>
#include <RobotAPI/libraries/aron/converter/json/NLohmannJSONConverter.h>

#include <RobotAPI/components/skills/SkillProviderExample/aron/HelloWorldAcceptedType.aron.generated.h>

namespace armarx::skills::provider
{
    HelloWorldSkill::HelloWorldSkill() :
        Skill(SkillDescription{
            "HelloWorld",
            "This skill logs a message on ARMARX_IMPORTANT",
            {},
            1000,
            armarx::skills::Example::HelloWorldAcceptedType::ToAronType()
        })
    {}
    Skill::Status HelloWorldSkill::_execute(const aron::data::DictPtr& d, const CallbackT&)
    {
        ARMARX_IMPORTANT << "Hi, from the Hello World Skill.\n" <<
                            "I received the following data: \n" <<
                            aron::converter::AronNlohmannJSONConverter::ConvertToNlohmannJSON(d).dump(2) << "\n" <<
                            "(executed at: " << IceUtil::Time::now() << ")";
        return Skill::Status::Succeeded;
    }

    TimeoutSkill::TimeoutSkill() :
        Skill(SkillDescription{
            "Timeout",
            "This fails with timeout reached",
            {},
            1000,
            nullptr
        })
    {}
    Skill::Status TimeoutSkill::_execute(const aron::data::DictPtr& d, const CallbackT&)
    {
        int i = 0;
        while (!timeoutReached)
        {
            // do heavy work
            std::this_thread::sleep_for(std::chrono::milliseconds(200));

            // check if work is done
            if(i++ > 15)
            {
                ARMARX_IMPORTANT << "Somehow the timeout check succeceded....";
                return Skill::Status::Succeeded;
            }
        }
        return Skill::Status::TimeoutReached;
    }

    CallbackSkill::CallbackSkill() :
        Skill(SkillDescription{
            "ShowMeCallbacks",
            "This skill does shows callbacks",
            {},
            1000,
            nullptr
        })
    {}
    Skill::Status CallbackSkill::_execute(const aron::data::DictPtr& d, const CallbackT& callback)
    {
        ARMARX_IMPORTANT << "Logging three updates via the callback";
        auto up1 = std::make_shared<aron::data::Dict>();
        up1->addElement("updateInfo", std::make_shared<aron::data::String>("Update 1"));
        callback(up1);

        auto up2 = std::make_shared<aron::data::Dict>();
        up2->addElement("updateInfo", std::make_shared<aron::data::String>("Update 2"));
        callback(up2);

        auto up3 = std::make_shared<aron::data::Dict>();
        up3->addElement("updateInfo", std::make_shared<aron::data::String>("Update 3"));
        callback(up3);

        return Skill::Status::Succeeded;
    }


    SkillProviderExample::SkillProviderExample() :
        SkillProviderComponentPluginUser()
    {}

    armarx::PropertyDefinitionsPtr SkillProviderExample::createPropertyDefinitions()
    {
        armarx::PropertyDefinitionsPtr defs = new ComponentPropertyDefinitions(getConfigIdentifier());
        return defs;
    }

    std::string SkillProviderExample::getDefaultName() const
    {
        return "SkillProviderExample";
    }

    void SkillProviderExample::onInitComponent()
    {
        // Add example skill
        addSkill(std::make_shared<HelloWorldSkill>());

        // Add another lambda example skill
        {
            skills::SkillDescription fooDesc;
            fooDesc.acceptedType = nullptr; // accept everything
            fooDesc.description = "This skill does exactly nothing.";
            fooDesc.skillName = "Foo";
            fooDesc.timeoutMs = 1000;
            addSkill([](const aron::data::DictPtr&){ std::cout << "Hello from Foo" << std::endl; return true; }, fooDesc);
        }

        // Add another example skill
        addSkill(std::make_shared<CallbackSkill>());

        // Add timeout skill
        addSkill(std::make_shared<TimeoutSkill>());
    }

    void SkillProviderExample::onConnectComponent()
    {

    }

    void SkillProviderExample::onDisconnectComponent()
    {

    }

    void SkillProviderExample::onExitComponent()
    {

    }
}
