#pragma once

#include <map>
#include <mutex>
#include <optional>
#include <string>

#include "../filter/frequencyFilter/FrequencyFilter.h"
#include "../extractor/imageExtractor/ImageExtractor.h"
#include "../converter/dict/json/JsonConverter.h"
#include "../converter/image/png/PngConverter.h"

#include <RobotAPI/libraries/armem/core/MemoryID.h>

namespace armarx::armem::server::ltm
{
    /// all necessary classes to convert an entry of the ltm to some other format(s)
    struct LongTermMemoryPipeline
    {
        // Memory Filters
        /// filter based on update frequency (removes full updates)
        filter::MemoryFrequencyFilterPtr memFreqFilter;

        // Snapshot filters
        /// filter based on latest snapshot stored (per entity)
        filter::SnapshotFrequencyFilterPtr snapFreqFilter;

        // Extractors
        /// Extract NDArrays with 2 dims and at least 10 values (Images)
        extractor::ImageExtractorPtr imageExtractor;

        // Converters
        /// Convert dicts (everything that has not been extracted) to JSON
        converter::dict::JsonConverterPtr dictConverter;
        /// Convert images to png
        converter::image::PngConverterPtr imgConverter;

    };

    /// @brief Interface functions for the longterm memory classes
    class MemoryItem
    {
    public:
        MemoryItem() = default;
        MemoryItem(const MemoryID&);
        MemoryItem(const MemoryID&, const LongTermMemoryPipeline&);
        virtual ~MemoryItem() = default;

        MemoryID id() const;
        std::string name() const;

        virtual void setMemoryID(const MemoryID&);

    protected:
        LongTermMemoryPipeline pipeline;

    private:
        MemoryID _id;
    };
} // namespace armarx::armem::server::ltm
